/* vi: set sw=4 ts=4: */
/*
 * pwent.c - This file is part of the libc-8086/pwd package for ELKS,
 * Copyright (C) 1995, 1996 Nat Friedman <ndf@linux.mit.edu>.
 * Copyright (C) 2001-2003 Erik Andersen <andersee@debian.org>
 * 
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
 * setpwent(), endpwent(), and getpwent() are included in the same object
 * file, since one cannot be used without the other two, so it makes sense to
 * link them all in together.
 */

#define _GNU_SOURCE
#include <features.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <paths.h>
#include "config.h"

#ifdef __UCLIBC_HAS_THREADS__
#include <pthread.h>
static pthread_mutex_t mylock = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
# define LOCK   pthread_mutex_lock(&mylock)
# define UNLOCK pthread_mutex_unlock(&mylock);
#else       
# define LOCK
# define UNLOCK
#endif      

/* file descriptor for the password file currently open */
static int pw_fd = -9;

void setpwent(void)
{
	LOCK;
	if (pw_fd > -1)
		close(pw_fd);
	pw_fd = open(_PATH_PASSWD, O_RDONLY);
	UNLOCK;
}

void endpwent(void)
{
	LOCK;
	if (pw_fd > -1)
		close(pw_fd);
	pw_fd = -9;
	UNLOCK;
}

int getpwent_r (struct passwd *password, char *buff, 
		size_t buflen, struct passwd **result)
{
	int ret=EINVAL;
	*result = NULL;

	LOCK;
	/* Open /etc/passwd if not yet opened */
	if (pw_fd == -9) {
		setpwent();
	}
	if (pw_fd == -1) {
		UNLOCK;
		return -1;
	}
	ret=__getpwent_r(password, buff, buflen, pw_fd);
	if (ret == 0) {
		UNLOCK;
		*result = password;
		return 0;
	}
	UNLOCK;
	__set_errno(ret);
	return ret;
}

struct passwd *getpwent(void)
{
	int ret;
	struct passwd *result;
	static struct passwd pwd;
	static char line_buff[PWD_BUFFER_SIZE];

	ret = getpwent_r(&pwd, line_buff, sizeof(line_buff), &result);
	if (ret == 0) {
		return &pwd;
	}
	__set_errno(ret);
	return NULL;
}

